# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Helpers": "Helpers.ipynb",
         "notify": "Helpers.ipynb",
         "keys": "Helpers.ipynb",
         "scanDb": "Helpers.ipynb",
         "toDf": "Helpers.ipynb",
         "fromSeries": "Helpers.ipynb",
         "fromDf": "Helpers.ipynb",
         "fromDict": "Helpers.ipynb",
         "toListDict": "Helpers.ipynb",
         "toSeries": "Helpers.ipynb",
         "setNoUpdate": "Helpers.ipynb",
         "setUpdate": "Helpers.ipynb",
         "DATABASE_TABLE_NAME": "database.ipynb",
         "INVENTORY_BUCKET_NAME": "database.ipynb",
         "INPUT_BUCKET_NAME": "database.ipynb",
         "REGION": "database.ipynb",
         "ACCESS_KEY_ID": "database.ipynb",
         "SECRET_ACCESS_KEY": "database.ipynb",
         "LINEKEY": "database.ipynb",
         "createIndex": "KeySchema.ipynb",
         "KeySchema": "KeySchema.ipynb",
         "DBHASHLOCATION": "database.ipynb",
         "DBCACHELOCATION": "database.ipynb",
         "ALLDATAKEY": "S3.ipynb",
         "DEFAULTKEYS": "S3.ipynb",
         "S3Cache": "S3.ipynb",
         "loadFromCache": "S3.ipynb",
         "saveRemoteCache": "S3.ipynb",
         "resetS3Cache": "S3.ipynb",
         "BRANCH": "Update.ipynb",
         "VALUEUPDATESCHEMAURL": "Update.ipynb",
         "Updater": "Update.ipynb",
         "updateWithDict": "Update.ipynb",
         "ValueUpdate2": "Update.ipynb",
         "saveBatchToDynamodb": "Update.ipynb",
         "saveBatchToDynamodb2": "Update.ipynb",
         "valueUpdate2": "Update.ipynb",
         "INTCOLS": "database.ipynb",
         "ProductDatabase": "database.ipynb",
         "cacheDb": "database.ipynb",
         "lambdaDumpToS3": "database.ipynb",
         "Product": "database.ipynb",
         "ValueUpdate": "database.ipynb",
         "chunks": "database.ipynb",
         "lambdaUpdateProduct": "database.ipynb",
         "updateS3Input": "database.ipynb",
         "lambdaUpdateS3": "database.ipynb",
         "ProductsFromList": "query.ipynb",
         "lambdaProductsFromList": "database.ipynb",
         "lambdaSingleQuery": "database.ipynb",
         "lambdaAllQuery": "database.ipynb",
         "lambdaAllQueryFeather": "database.ipynb",
         "Querier": "query.ipynb",
         "allQuery": "query.ipynb",
         "productsFromList": "query.ipynb",
         "singleProductQuery": "query.ipynb"}

modules = ["helpers.py",
           "schema.py",
           "s3.py",
           "update.py",
           "database.py",
           "query.py"]

doc_url = "https://thanakijwanavit.github.io/villaProductDatabase/"

git_url = "https://github.com/thanakijwanavit/villaProductDatabase/tree/master/"

def custom_doc_links(name): return None
